#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif
#include <math.h>
#include "ai.h"
#include "map.h"
#include "playerpool.h"
#include "unit.h"
#include "structure.h"
#include "unitorstructure.h"
#include "weaponspool.h"
#include "logger.h"
#include "memtrack.h"

using namespace std;

UnitAndStructurePool* AI::uspool;

void AI::setUSPool(UnitAndStructurePool* usp)
{
    uspool = usp;
}

AI::AI(CnCMap* map, Player* player)
{
    this->map = map;
    ppool = map->getPlayerPool();
    this->player = player;
    mapwidth = map->getWidth();
    attacktype = DEFAULT_ATTACK;
    prefs.inf_inf = 1.0f;
    prefs.veh_inf = 1.2f;
    prefs.inf_veh = 1.4f;
    prefs.veh_veh = 1.0f;
    prefs.unarmed = 2.0f;
}

void AI::printAttackType()
{
    logger->indent();
    if ((attacktype & ATTACK_WITH_ALL) == ATTACK_WITH_ALL) {
        logger->note("do attack with all\n");
    } else {
        logger->note("do not attack with all\n");
    }
    if ((attacktype & ATTACK_ONE_ON_ONE) == ATTACK_ONE_ON_ONE) {
        logger->note("do attack one on one\n");
    } else {
        logger->note("do not attack one on one\n");
    }
    if ((attacktype & ATTACK_NEAREST) == ATTACK_NEAREST) {
        logger->note("do attack nearest\n");
    } else {
        logger->note("do not attack nearest\n");
    }
    if ((attacktype & ATTACK_MATCHED_TYPE) == ATTACK_MATCHED_TYPE) {
        logger->note("do attack matched type\n");
    } else {
        logger->note("do not attack matched type\n");
    }
    logger->unindent();
}

void AI::toggleAttackType(Uint16 attacktype)
{
    attacktype ^= attacktype;
    printAttackType();
}

/** Finds a suitable opponent for each unit and structure for a player.
 * 
 * @bug This is hackish: units and structures should ask for targets except when
 * the "alltoattack" map trigger is activated.
 *
 * The algorithm for selecting the best target for a given unit or structure goes
 * like this (we assume more than one opponent):
 *!
 * -# set minpri to mapwidth^4 (magic number, don't know why it's used)
 * -# For each of the opponents' units and structures, compute the distance
 * between it and the attacker.
 * -# If the attacker is a structure and the current target is out of range of
 * the attacker's weapons, move onto the next unit or structure of the oppenents
 * immediately.
 * -# Multiply this distance by a constant.  The constant is different whether
 * the attacker is infantry or not, whether the target being considerred is
 * infantry or not, and whether the target is unarmed or not.
 * -# If this value is less than minpri, set minpri to this and store the
 * current target.
 * -# Repeat from step 1 until considerred all possible targets.
 * -# If we found a target, tell the attacker to attack it.
 * -# If we are to attack one-on-one style, remove this target from the
 * available targets.
 */
void AI::attackOtherSide()
{
    Uint16 i,j;
    Player* opponent = NULL;
    float pri, minpri;
    vector<Player*> opps = ppool->getOpponents(player);
    vector<UnitOrStructure *> attackers, targets;
    vector<Unit *> units, opp_units;
    vector<Structure*> structs, opp_structs;
    UnitOrStructure* target;
    Uint16 targetno = 0;
    units = player->getUnits();
    structs = player->getStructures();
    // get all attacking units
    for (i = 0 ; i < units.size() ; ++i) {
        if (units[i]->isAlive() && units[i]->canAttack()) {
            attackers.push_back(units[i]);
        }
    }
    for (i = 0 ; i < structs.size() ; ++i) {
        if (structs[i]->isAlive() && structs[i]->canAttack()) {
            attackers.push_back(structs[i]);
        }
    }

    // get all units or structures that can be attacked
    if (opps.size() > 0) {
        opponent = opps[0];
    }
    for (i = 0 ; i < opps.size() ; ++i, opponent = opps[i]) {
        opp_units = opponent->getUnits();
        // all target units
        for (j = 0 ; j < opp_units.size(); ++j) {
            if (opp_units[j]->isAlive()) {
                targets.push_back(opp_units[j]);
            }
        }
        opp_structs = opponent->getStructures();
        // all target structures
        for (j = 0 ; j < opp_structs.size(); ++j) {
            if (opp_structs[j]->isAlive()) {
                targets.push_back(opp_structs[j]);
            }
        }
    }

    // find best target
    for (i = 0 ; i < attackers.size() ; ++i) {
        // determine coordinates
        minpri = mapwidth * mapwidth * mapwidth * mapwidth;
        target = NULL;
        for (j = 0 ; j < targets.size() ; ++j) {
            if (targets[j] == NULL) {
                continue;
            }
            pri = calcPri(attackers[i],targets[j]);
            /* special constant meaning attacker is a structure and
             * current target is out of range (so might never get in
             * range)
             */
            if (pri == 1000.0)
                continue;
            // found a better target
            if (pri < minpri) {
                target = targets[j];
                minpri = pri;
                targetno = j;
            }
        }
        if (target != NULL) {
            attackers[i]->attack(target);
            if ((attacktype & ATTACK_ONE_ON_ONE) == ATTACK_ONE_ON_ONE) {
                targets[targetno] = NULL;
            }
        }
    }
}

float AI::calcPri(UnitOrStructure* atk_uos, UnitOrStructure* trg_uos)
{
    Uint16 ax, ay, tx, ty;
    ax = atk_uos->getPos() % mapwidth;
    ay = atk_uos->getPos() / mapwidth;
    tx = trg_uos->getPos() % mapwidth;
    ty = trg_uos->getPos() / mapwidth;
    bool trg_armed = (trg_uos->getType()->getWeapon(true) != NULL);
    bool atk_isinf = atk_uos->getType()->isInfantry();
    bool trg_isinf = trg_uos->getType()->isInfantry();
    float dist, measure;
    dist = 0;
    if ((attacktype & ATTACK_NEAREST) == ATTACK_NEAREST) {
        dist = sqrt((ax - tx) * (ax - tx) +
                    (ay - ty) * (ay - ty));
    }
    if (atk_uos->getType()->getSpeed() == 0) {
        // structures can't move, so this target can be discarded
        if (dist > atk_uos->getType()->getWeapon(true)->getRange())
            return 1000.0;
    }
    measure = 1.00f;
    if ((attacktype & ATTACK_MATCHED_TYPE) == ATTACK_MATCHED_TYPE) {
        if (!trg_armed) {
            measure *= prefs.unarmed;
        }
        if (atk_isinf) {
            if (trg_isinf) {
                measure *= prefs.inf_inf;
            } else {
                measure *= prefs.inf_veh;
            }
        } else {
            if (trg_isinf) {
                measure *= prefs.veh_inf;
            } else {
                measure *= prefs.veh_veh;
            }
        }
    }
    if (DEBUG_LEVEL < 2)
        logger->debug("dist: %d simil: %d\n", dist, measure);
    return (dist * measure);
}

void AI::optimise()
{}

void AI::lostUnit(Unit* Unit)
{}

void AI::lostStructure(Structure* Structure)
{}

void AI::lostUnitOrStructure(UnitOrStructure* UnitOrStructure)
{}

