#include "netconnection.h"
#include "logger.h"
#include "common.h"
#include "memtrack.h"

NetConnection::NetConnection(char *address, Uint16 port)
{

#ifndef NONET
    if(SDLNet_Init()==-1) {
        logger->error("SDLNet_Init: %s\n", SDLNet_GetError());
        throw 0;
    }

    SDLNet_ResolveHost(&ip, address, port);
    tcpsock = SDLNet_TCP_Open(&ip);
    if (!tcpsock) {
        logger->error("Can't connect. Reason : %s\n", SDLNet_GetError());
        throw 0;
    }
#else
    logger->error("This version doesn't support networking, plase recompile\n");
    throw 0;
#endif

}

NetConnection::~NetConnection()
{
#ifndef NONET
    SDLNet_TCP_Close(tcpsock);
#endif
}

void NetConnection::login(const char* version, const char* nick, const char* side, const char* colour)
{
#ifndef NONET
    int result, len, a;
    char* tmp;
    tmp = new char[128];
    logger->note("sending data package : ");
    for (a=1; a<5; a++) {
    	switch (a) {
    		case '1'	:	sprintf(tmp, "%s%s\n", "version:", version); break;
		case '2'	:       sprintf(tmp, "%s%s\n", "nick:", nick); break;
		case '3'	:       sprintf(tmp, "%s%s\n", "side:", side); break;
		case '4'	:	sprintf(tmp, "%s%s\n", "colour:", colour); break;
        }
        len = strlen(tmp) + 1;
        //tmp[len-1]='\0';
        result = SDLNet_TCP_Send(tcpsock, tmp, len);
        if (result < len) {
            logger->warning("socket : failure in sending.\n");
        }
        logger->note("%d ", a);
    }
    delete[] tmp;
#endif
}

void NetConnection::logout()
{
#ifndef NONET
    char *terminate = NULL;
    int len;
    strcpy(terminate, "terminates\n");
    len = strlen(terminate)+1;
    send((Uint8 *)terminate, len);
#endif
}

Uint16 NetConnection::send(Uint8 *data, Uint16 len)
{
#ifndef NONET
    return SDLNet_TCP_Send(tcpsock, data, len);
#else

    return 0;
#endif
}

Uint16 NetConnection::recv(Uint8 *data, Uint16 len)
{
#ifndef NONET
    return SDLNet_TCP_Recv(tcpsock, data, len);
#else

    return 0;
#endif
}

