#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif
#include <string>
#include <ctime>
#include "timeline.h"
#include "playerpool.h"
#include "unit.h"
#include "structure.h"
#include "logger.h"
#include "common.h"
#include "memtrack.h"

//pander need to work on error control

using namespace std;

int getPlotColour(const char *pname)
{
    char x;
    if (strcasecmp(pname, "badguy") == 0) {
        return 1;
    } else if (strcasecmp(pname, "goodguy") == 0) {
        return 2;
    } else if (strcasecmp(pname, "neutral") == 0) {
        return 3;
    } else if (strcasecmp(pname, "special") == 0) {
        return 4;
    } else if (strncasecmp(pname, "multi",5) == 0) {
        x = pname[5];
        return atoi(&x);
    }
    return -1;
}

PlayerStatistics::PlayerStatistics(Player *player)
{
    this->player = player;
}

PlayerStatistics::~PlayerStatistics()
{}

const char* PlayerStatistics::getPlayerName()
{
    return player->getName();
}

void PlayerStatistics::update()
{
    money.push_back(player->getMoney());
    power.push_back(player->getPower());
    powerused.push_back(player->getPowerUsed());
    numunits.push_back(player->getNumUnits());
    numstructs.push_back(player->getNumStructs());
}

void PlayerStatistics::dumpstats(FILE* f, Uint16 i)
{
    fprintf(f,"\t%d",numunits[i]);
    fprintf(f,"\t%d",numstructs[i]);
}

TimeLine::TimeLine(PlayerPool *PlayerPool)
{
    for (Uint8 i = 0; i < PlayerPool->getNumPlayers();++i) {
        pstats.push_back(new PlayerStatistics(PlayerPool->getPlayer(i)));
    }
    starttime = (Uint32)time((time_t*)NULL);
}

TimeLine::~TimeLine()
{
    Uint16 i;
    for (i=0;i<pstats.size();++i) {
        delete pstats[i];
    }
}

void TimeLine::update(PlayerPool *PlayerPool)
{
    if (PlayerPool->getNumPlayers() != pstats.size()) {
        logger->error("TimeLine:  number of players has changed\n");
        return;
    }
    times.push_back((Uint32)time((time_t*)NULL)-starttime);
    for (Uint8 i=0; i < pstats.size(); ++i) {
        pstats[i]->update();
    }
}

/** @brief dumps stats to a file that gnuplot can read.
 * @bug not portable.
 * @bug uses system().
 * @bug uses which to detect availability.
 * @bug works with only gnuplot.
 */
void TimeLine::plot()
{
#if !defined _WIN32
    FILE* plotscr, *plotdata;
    Uint16 i,j;
    bool first = true;
    if (system("which gnuplot 1>/dev/null 2>/dev/null")) {
        logger->error("TimeLine: gnuplot not availble\n");
        return;
    }
    if (!(plotscr = fopen("timeline.plt", "w"))) {
        logger->error("TimeLine: could not open file for writing timeline.plt");
        return;
    }
    fprintf(plotscr, "set title \"FreeCNC time line\"\n");
    fprintf(plotscr, "set xlabel \"t [s]\"\n");
    fprintf(plotscr, "plot");
    i = 2;
    for (j=0; j < pstats.size(); ++j) {
        const char *pname = pstats[j]->getPlayerName();
        if (first) {
            fprintf(plotscr, " \"timeline.dat\" using 1:%d t \"%s units\" w lp lt %d pt 4",
                    i, pname, getPlotColour(pname));
            first = false;
        } else {
            fprintf(plotscr, ", \"timeline.dat\" using 1:%d t \"%s units\" w lp lt %d pt 4",
                    i, pname, getPlotColour(pname));
        }
        ++i;
        fprintf(plotscr, ", \"timeline.dat\" using 1:%d t \"%s structs\" w lp lt %d pt 3",
                i, pname, getPlotColour(pname));
        ++i;
    }
    fprintf(plotscr, ";pause 3\n");
    fclose(plotscr);
    if (!(plotdata = fopen("timeline.dat", "w"))) {
        logger->error("TimeLine: could not open file for writing timeline.dat\n");
        return;
    }
    fprintf(plotdata, "#Time\t");
    for (i = 0; i < pstats.size();++i) {
        fprintf(plotdata, "%s\t", pstats[i]->getPlayerName());
    }
    fprintf(plotdata, "\n");
    for (j = 0; j < times.size();++j) {
        fprintf(plotdata, "%d", times[j]);
        for (i = 0; i < pstats.size();++i) {
            pstats[i]->dumpstats(plotdata,j);
        }
        fprintf(plotdata, "\n");
    }
    fclose(plotdata);
    if (system("gnuplot timeline.plt 1>/dev/null 2>/dev/null &")) {
        logger->error("TimeLine: gnuplot could not plot timeline.plt\n");
        return;
    }
#endif
}
