#include <algorithm>
#include <ctype.h>
#include "weaponspool.h"
#include "inifile.h"
#include "shpimage.h"
#include "unit.h"
#include "structure.h"
#include "unitandstructurepool.h"
#include "projectileanim.h"
#include "game.h"
#include "unitorstructure.h"
#include "logger.h"
#include "memtrack.h"

using namespace std;

Warhead::Warhead(const char *whname, INIFile *weapini, WeaponsPool *wp)
{
    char *tmpname;
    SHPImage* temp;

    tmpname = weapini->readString(whname, "explosionimage");
    explosionimage = 0;
    if( tmpname != NULL ) {
        explosionimage = wp->imgpool->size()<<16;
        try {
            temp = new SHPImage(tmpname, Game::mapscaleq);
        } catch (ImageNotFound* e) {
            delete e;
#if defined _DEBUG && !defined _WIN32

            RemoveTrack((unsigned int)this,false);
#endif

            throw 0;
        }
        wp->imgpool->push_back(temp);
        delete[] tmpname;
    }
    explosionanimsteps = temp->getNumImg();
    explosionsound = weapini->readString(whname, "explosionsound");
    infantrydeath = 0;
    walls = (weapini->readInt(whname, "walls",0) != 0);
    trees = false;
    //blastradius = 0;
    tmpname = weapini->readString(whname, "versus");
    versus[0] = 100;
    versus[1] = 100;
    versus[2] = 100;
    versus[3] = 100;
    versus[4] = 100;
    if (tmpname != NULL) {
        sscanf(tmpname,"%u,%u,%u,%u,%u",&versus[0],&versus[1],
               &versus[2],&versus[3],&versus[4]);
    }
    delete[] tmpname;
    //for (int i=0;i<5;++i)
    //    fprintf(stderr,"%s\t%i\t%i\n",whname,i,versus[i]);
}

Warhead::~Warhead()
{
    if( explosionsound != NULL ) {
        delete[] explosionsound;
    }
}

Projectile::Projectile(const char *pname, INIFile *weapini, WeaponsPool *wp)
{
    char *iname = weapini->readString(pname, "image");
    SHPImage* temp;
    imagenum = 0;
    rotates = false;
    rotationimgs = 0;
    if( iname != NULL ) {
        imagenum = wp->imgpool->size()<<16;
        try {
            temp = new SHPImage(iname, Game::mapscaleq);
        } catch (ImageNotFound* e) {
            delete e;
#if defined _DEBUG && !defined _WIN32

            RemoveTrack((unsigned int)this,false);
#endif

            throw 0;
        }
        //  printf("Projectile %s has %s which has %i\n",pname,iname,temp->getNumImg());
        wp->imgpool->push_back(temp);
        delete[] iname;

        if (weapini->readInt(pname,"rotates",0) != 0) {
            rotationimgs = temp->getNumImg();
            rotates = true;
        }
    }
    AA = false;
    AG = true;
    high = false;
    inacurate = false;
}

Projectile::~Projectile()
{}

Weapon::Weapon(const char *wname, WeaponsPool *wp)
{
    char *pname, *whname, *faname, *faimage;
    map<string, Projectile*>::iterator projentry;
    map<string, Warhead*>::iterator wheadentry;
    INIFile *weapini = wp->getWeaponsINI();
    SHPImage* fireanimtemp;
    Uint8 additional, i;
    std::string projname, warheadname;
    std::string weapname = (string)wname;
    std::string::iterator p = weapname.begin();
    while (p!=weapname.end())
        *p++ = toupper(*p);

    this->wp = wp;
    pname = weapini->readString(wname, "projectile");
    projname = (string)pname;
    p = projname.begin();
    while (p!=projname.end())
        *p++ = toupper(*p);

    if( pname == NULL ) {
        logger->warning("Unable to find projectile for weapon \"%s\" in inifile..\n", wname);
#if defined _DEBUG && !defined _WIN32

        RemoveTrack((unsigned int)this,false);
#endif

        throw 0;
    }
    projentry = wp->projectilepool.find(projname);
    if( projentry == wp->projectilepool.end() ) {
        try {
            projectile = new Projectile(pname, weapini, wp);
        } catch( int e ) {
            logger->warning("Unable to find projectile \"%s\" used for weapon \"%s\".\nUnit using this weapon will be unarmed\n", pname, wname);
            delete[] pname;
#if defined _DEBUG && !defined _WIN32

            RemoveTrack((unsigned int)this,false);
#endif

            throw 0;
        }
        wp->projectilepool[projname] = projectile;
    } else {
        projectile = projentry->second;
    }
    delete[] pname;

    whname = weapini->readString(wname, "warhead");
    if( whname == NULL ) {
        logger->warning("Unable to find warhead for weapon \"%s\" in inifile..\n", wname);
#if defined _DEBUG && !defined _WIN32

        RemoveTrack((unsigned int)this,false);
#endif

        throw 0;
    }
    warheadname = (string)whname;
    transform(warheadname.begin(),warheadname.end(), warheadname.begin(), toupper);
    wheadentry = wp->warheadpool.find(warheadname);
    if( wheadentry == wp->warheadpool.end() ) {
        try {
            whead = new Warhead(whname, weapini, wp);
        } catch( int ) {
            logger->warning("Unable to find Warhead \"%s\" used for weapon \"%s\".\nUnit using this weapon will be unarmed\n", whname, wname);
            delete[] whname;
#if defined _DEBUG && !defined _WIN32

            RemoveTrack((unsigned int)this,false);
#endif

            throw 0;
        }
        wp->warheadpool[warheadname] = whead;
    } else {
        whead = wheadentry->second;
    }
    delete[] whname;

    speed      = weapini->readInt(wname, "speed", 100);
    range      = weapini->readInt(wname, "range", 1);
    reloadtime = weapini->readInt(wname, "reloadtime", 5);
    damage     = weapini->readInt(wname, "damage", 10);
    burst      = weapini->readInt(wname, "burst", 1);
    heatseek   = (weapini->readInt(wname, "heatseek", 0) != 0);
    inaccurate = (weapini->readInt(wname, "inaccurate", 0) != 0);
    fireimage  = wp->imgpool->size()<<16;
    //    wp->imgpool->push_back(new SHPImage("minigun.shp", Game::mapscaleq));
    firesound  = weapini->readString(wname, "firesound");
    fuel       = weapini->readInt(wname, "fuel", 0);
    seekfuel   = weapini->readInt(wname, "seekfuel", 0);

    faname = weapini->readString(wname, "fireimage", "none");
    if (strcasecmp(faname,"none") == 0) {
        delete[] faname;
        numfireimages = 0;
        numfiredirections = 1;
        fireimage = 0;
    } else {
        additional = (Uint8)weapini->readInt(faname,"additional",0);
        faimage = weapini->readString(faname, "image", "minigun.shp");
        try {
            fireanimtemp = new SHPImage(faimage, Game::mapscaleq);
        } catch (ImageNotFound* e) {
            delete e;
#if defined _DEBUG && !defined _WIN32

            RemoveTrack((unsigned int)this,false);
#endif

            throw 0;
        }
        delete[] faimage;
        faimage = NULL;
        numfireimages = fireanimtemp->getNumImg();
        numfiredirections = weapini->readInt(faname, "directions", 1);
        if (numfiredirections == 0) {
            numfiredirections = 1;
        }
        fireimages = new Uint32[numfiredirections];
        fireimages[0] = fireimage;
        wp->imgpool->push_back(fireanimtemp);
        if (additional != 0) {
            char* tmpname = new char[12];
            for (i=2;i<=additional;++i) {
                sprintf(tmpname,"image%i",i);
                faimage = weapini->readString(faname, tmpname, "");
                if (strcasecmp(faimage,"") != 0) {
                    try {
                        fireanimtemp = new SHPImage(faimage, Game::mapscaleq);
                    } catch (ImageNotFound* e2) {
                        delete e2;
#if defined _DEBUG && !defined _WIN32

                        RemoveTrack((unsigned int)this,false);
#endif

                        throw 0;
                    }
                    fireimages[i-1]=(wp->imgpool->size()<<16);
                    numfireimages += fireanimtemp->getNumImg();
                    wp->imgpool->push_back(fireanimtemp);
                } else {
                    fireimages[i] = 0;
                    logger->warning("%s was empty in [%s]\n",tmpname,faname);
                }
                delete[] faimage;
                faimage = NULL;
            }
            delete[] tmpname;
        } else if (numfiredirections != 1) {
            for (i=1;i<numfiredirections;++i) {
                fireimages[i] = fireimage+i*(numfireimages/numfiredirections);
            }
        }
        if (faname != NULL) {
            delete[] faname;
        }
    }
}

Weapon::~Weapon()
{
    if( firesound != NULL ) {
        delete[] firesound;
    }
    if (fireimage != 0) {
        delete[] fireimages;
    }
}

void Weapon::fire(UnitOrStructure *owner, Uint16 target, Uint8 subtarget, UnitAndStructurePool *uspool)
{
    if( firesound != NULL ) {
        uspool->getsfxeng()->queueSound(firesound);
    }
    if( fireimage != 0 ) {
        Uint32 length = numfireimages;
        Uint8 facing;
        if (owner->getType()->getNumLayers() == 1) {
            facing = (owner->getImageNum(0))&0x1f;
        } else {
            facing = (owner->getImageNum(1))&0x1f;
        }
        if (!owner->getType()->isInfantry()) {
            facing >>= 2;
        }
        length /= numfiredirections;
        if (numfiredirections == 1) {
            facing = 0;
        }
        new ExplosionAnim(1, uspool, owner->getPos(),fireimages[facing],
                          (Uint8)length,/*owner->getXoffset()+*/InfantryGroup::getunitoffsets()[owner->getSubpos()],
                          /*owner->getYoffset()+*/InfantryGroup::getunitoffsets()[owner->getSubpos()]);
    }
    uspool->scheduleUSEvent(new ProjectileAnim(0, this, owner, target, subtarget, uspool));
}

WeaponsPool::WeaponsPool(vector<SHPImage*> *imgpool)
{
    this->imgpool = imgpool;
    weapini = new INIFile("weapons.ini");
}

WeaponsPool::~WeaponsPool()
{
    delete weapini;
    std::map<std::string, Weapon*>::const_iterator wpclear;
    std::map<std::string, Warhead*>::const_iterator whpclear;
    std::map<std::string, Projectile*>::const_iterator ppclear;
    wpclear  = weaponspool.begin();
    whpclear = warheadpool.begin();
    ppclear  = projectilepool.begin();

    while (wpclear != weaponspool.end()) {
        if (wpclear->second != NULL) {
            delete wpclear->second;
        }
        ++wpclear;
    }
    while (whpclear != warheadpool.end()) {
        if (whpclear->second != NULL) {
            delete whpclear->second;
        }
        ++whpclear;
    }
    while (ppclear != projectilepool.end()) {
        if (ppclear->second != NULL) {
            delete ppclear->second;
        }
        ++ppclear;
    }
}

Weapon *WeaponsPool::getWeapon(const char *wname)
{
    map<string, Weapon*>::iterator weapentry;
    Weapon *weap;
    std::string weapname = (string)wname;
    transform(weapname.begin(),weapname.end(), weapname.begin(), toupper);
    weapentry = weaponspool.find(weapname);

    if( weapentry == weaponspool.end() ) {
        try {
            weap = new Weapon(wname, this);
        } catch( int ) {
            /* if weapon is NULL unit doesn't have a weapon */
            return NULL;
        }
        weaponspool[weapname] = weap;
        return weap;
    } else {
        return weapentry->second;
    }
}
