// mode: -*- C++ -*-
/*****************************************************************************
 * ai.h - header file for AI class
 *
 ****************************************************************************/

#ifndef __AI_H
#define __AI_H

#include "SDL_types.h"

class CnCMap;
class UnitAndStructurePool;
class PlayerPool;
class Player;
class Unit;
class Structure;
class UnitOrStructure;

#define DEBUG_LEVEL 4

// attack type
#define ATTACK_WITH_ALL 0x1
#define ATTACK_ONE_ON_ONE 0x2
#define ATTACK_NEAREST 0x4
#define ATTACK_MATCHED_TYPE 0x8
#define ATTACK_CONSIDER_SUBPOS 0x16
#define DEFAULT_ATTACK ATTACK_WITH_ALL | ATTACK_NEAREST | ATTACK_MATCHED_TYPE | ATTACK_CONSIDER_SUBPOS
//#define DEFAULT_ATTACK ATTACK_WITH_ALL | ATTACK_ONE_ON_ONE | ATTACK_NEAREST | ATTACK_MATCHED_TYPE | ATTACK_CONSIDER_SUBPOS

/**
 * table of constants used to determine the best target for an attacker.
 * format: attacker_target
 * lower values mean higher priority.
 */
struct AIPrefs
{
    float inf_inf, veh_inf, inf_veh, veh_veh;
    float unarmed;
};

/** @brief Finds suitable targets for units and tells them to attack
 * 
 * @author Main code: Sander van Geloven, 2002.
 * Cleanups and commenting by Euan MacGregor.
 *
 * @todo Construction/base management.
 *
 * @todo Make issuing attack orders not such a hack. 
 *
 * @todo Add a value for whether to prefer/avoid attacking structures
 * (would need some way of determining if the UnitOrStructure is in
 * fact a Structure or not).
 * 
 * @todo AIPrefs should vary per type (would reduce to four values: 
 * vs infantry, vs vehicles, vs unarmed and vs structures).
 */

class AI
{
public:
    AI(CnCMap* map, Player* player);
    ~AI();

    void printAttackType();
    void toggleAttackType(Uint16 attacktype);
    void attackOtherSide();
    float calcPri(UnitOrStructure* atk_uos, UnitOrStructure* trg_uos);
    void printInfo();
    void lostUnit(Unit* un);
    void lostStructure(Structure* str);
    void lostUnitOrStructure(UnitOrStructure* uos);
    AIPrefs* getPrefs()
    {
        return &prefs;
    }
    void optimise();
    static void setUSPool(UnitAndStructurePool* usp);
private:
    CnCMap* map;
    static UnitAndStructurePool* uspool;
    PlayerPool* ppool;
    Player *player;
    Uint16 attacktype;
    Uint16 mapwidth;
    AIPrefs prefs;
};

#endif
