// mode: -*- C++ -*-
#ifndef __COMMON_H__
#define __COMMON_H__

/* This file contains data types and constants that are used by multiple
 * classes, in possibly different files
 */

#ifdef _MSC_VER
#pragma warning (disable: 4503)
#pragma warning (disable: 4786)
#endif

#include "SDL_types.h"
#include <cstring>
#include <vector>
#include "memtrack.h"

#define VERSION "0.2.1-CVS"
#define MAXPLAYERS 6
#define GAME_TD 1
#define GAME_RA 2

#ifndef MIN
#define MIN(x,y) (((x)<(y))?(x):(y))
#endif

#ifndef MAX
#define MAX(x,y) (((x)>(y))?(x):(y))
#endif

/* From math.h when __USE_BSD || defined __USE_XOPEN is defined
 * Not sure what is needed for MSVC though, so I'm sticking these here
 * for now
 */
#define M_PI   3.14159265358979323846
#define M_PI_2 1.57079632679489661923

struct animinfo_t
{
    Uint32 animdelay;
    Uint8 loopend, loopend2, animspeed, animtype, sectype;
    Uint8 dmgoff, dmgoff2;
    Uint16 makenum;
};

struct powerinfo_t
{
    Uint16 power;
    Uint16 drain;
    bool powered;
};


enum PSIDE {
    PS_UNDEFINED = 0, PS_GOOD = 0x1, PS_BAD = 0x2,
    PS_NEUTRAL = 0x4, PS_SPECIAL = 0x8, PS_MULTI = 0x10
};

enum armour_t {
    AC_none = 0, AC_wood = 1, AC_light = 2, AC_heavy = 3, AC_concrete = 4
};

enum LOADSTATE {
    PASSENGER_NONE = 0, PASSENGER_LOAD = 1, PASSENGER_UNLOAD = 2
};

enum TalkbackType {
    TB_report, TB_ack, TB_atkun, TB_atkst, TB_die, TB_postkill, TB_invalid
};

#ifndef _WITHOUT_STRCASECMP
int strncasecmp(const char*, const char*, size_t);
int strcasecmp(const char*, const char*);
#endif

/// Same as strdup but uses C++ style allocation
inline char* cppstrdup(const char* s)
{
    char* r = new char[strlen(s)+1];
    return strcpy(r,s);
}

inline bool isRelativePath(const char *p)
{
#ifdef _WIN32
    return ((strlen(p) == 0) || p[1] != ':') && p[0] != '\\' && p[0] != '/';
#else

    return p[0] != '/';
#endif
}

std::vector<char*> splitList(char* line, char delim);

char* stripNumbers(const char* src);

char *determineBinaryLocation(const char *launchcmd);

#endif /* __COMMON_H__ */
