// mode: -*- C++ -*-
/*****************************************************************************
 * sidebar.h - header file for sidebar
 *
 ****************************************************************************/
#ifndef __SIDEBAR_H
#define __SIDEBAR_H

#include "SDL.h"
#include "actioneventqueue.h"
#include "font.h"
#include <vector>

using namespace std;

#define B_NULL 0
#define B_BUILD 1
#define B_SCROLL 2
#define B_UNIT 4
#define B_STRUCTURE 8
#define B_UP 16
#define B_DOWN 32

class Player;

class Sidebar;

class RadarAnimEvent : public ActionEvent
{
public:
    RadarAnimEvent(Uint32 p, Uint8 mode, Sidebar* sidebar, bool* minienable);
    virtual void run();
private:
    Uint8 mode, frame, framend;
    bool* minienable;
    Sidebar* sidebar;
};

class SidebarButton
{
public:
    SidebarButton(Sint16 x, Sint16 y, const char* fname, Uint8 f,
                  const char* theater, Sidebar* sb, Uint8 palnum);
    ~SidebarButton();
    bool ChangeImage(const char* fname);
    bool ChangeImage(const char* fname, Uint8 number);
    SDL_Surface* getSurface(void)
    {
        return pic;
    }
    SDL_Rect getRect(void)
    {
        return picloc;
    }
    Uint8 getFunction(void)
    {
        return function;
    }
private:
    void fallback(const char* fname);
    SDL_Surface *pic;
    SDL_Rect picloc;
    Uint8 function, palnum;
    const char* theater;
    Sidebar* sidebar;
};

class Sidebar
{
public:
    friend class RadarAnimEvent;

    Sidebar(ActionEventQueue* evQueue,
            Player *pl, Uint16 height, const char* theater);
    ~Sidebar();
    Uint8 getChanged();
    //   void setChanged(Uint8 ch){ changed = ch; };
    Uint8 getVisible()
    {
        return visible;
    };
    void toggleVisible();
    SDL_Surface *getTabImage()
    {
        return tab;
    };
    SDL_Rect *getTabLocation()
    {
        return &tablocation;
    };
    SDL_Surface *getSidebarImage(SDL_Rect location);
    Uint8 getButton(Uint16 x, Uint16 y);
    void clickButton(Uint8 index, char* unitname, Uint8* createmode);
    void resetButton(void);
    void startRadarAnim(Uint8 mode, bool* minienable);
    Font *getFont()
    {
        return gamefnt;
    }
    void scrollSidebar(bool scrollup);
    void updateSidebar();
    bool isOriginalType()
    {
        return isoriginaltype;
    }
private:
    // 1 - DOS, 0 - GOLD
    bool isoriginaltype;
    Uint8 spalnum;

    void setupbuttons(Uint16 height);
    void scrollbuildlist(Uint8 dir, Uint8 type);
    void build(Uint8 index, Uint8 type, char* unitname, Uint8* createmode);
    void updateicons(void);
    void updateAvailableLists(void);
    void downbutton(Uint8 index);
    void addButton(Uint16 x, Uint16 y, const char* fname, Uint8 f, Uint8 pal);

    SDL_Surface *tab;
    SDL_Rect tablocation;
    SDL_Surface *sbar;
    SDL_Rect sbarlocation;

    SDL_Surface *radarlogo;

    Font *gamefnt;

    Uint8 visible;
    Uint8 changed;         // check to see if we should redraw the whole screen

    const char* theater;

    Uint8 buttondown;
    bool bd;

    Uint8 buildbut;
    std::vector<SidebarButton *> buttons;

    std::vector<char*> uniticons;
    std::vector<char*> structicons;

    const char* radarname;
    RadarAnimEvent* radaranim;
    bool radaranimating;

    Uint8 unitoff,structoff; // For scrolling

    ActionEventQueue* evQueue;
    Player *player;
    int scaleq;
};

class SidebarError
    {}
;

#endif
