#ifdef _MSC_VER
# pragma warning (disable: 4503)
# pragma warning (disable: 4786)
#endif
#include "SDL_endian.h"
#include "vfs.h"
#include "inifile.h"
#include <vector>
#include <map>
#include <string>
#include <cstring>
#include <cstdarg>
#include "logger.h"
#include "archive.h"
#include "externalvfs.h"
#include "vfsplugman.h"
#include "common.h"
#include "memtrack.h"

using namespace std;

ExternalFiles *externals;
VFSPlugMan *plugins;

/** Sets up externals so that the logger can work
 */
void VFS_PreInit(const char* binpath)
{
    externals = new ExternalFiles(binpath);
}

/** @todo install prefix
 */
void VFS_Init(const char* binpath)
{
    Uint32 i;
    //int required;
    INIFile *filesini;
    char *defpath;
    char *game, *fname;
    char tempstr[1024];
    Uint32 keynum;
    //INIKey key;

    logger->note("Assuming FreeCNC is installed in \"%s\"\n", binpath);

    externals->loadArchive("conf/");
#if defined _WIN32
#elif defined(macintosh)
#else

    externals->loadArchive("/etc/freecnc/");
#endif

    try {
        filesini = new INIFile("files.ini");
    } catch(char *fname) {
        logger->error("Unable to locate \"%s\"\n", fname);
        exit(1);
    }
    i = 1;
    do {
        sprintf(tempstr, "path%d", i++);
        defpath = filesini->readString("general", tempstr);
        if( defpath != NULL ) {
            if( defpath[strlen(defpath)-1] != '/' &&
                    defpath[strlen(defpath)-1] != '\\' ) {
                strcpy(tempstr, defpath);
                strcat(tempstr, "/");
                delete[] defpath;
                defpath = cppstrdup(tempstr);
            }
            externals->loadArchive(defpath);
            delete[] defpath;
        }
    } while(defpath != NULL);
    /* load all the mixes here */
    for( i = 1;;i++ ) {
        plugins = new VFSPlugMan(binpath);
        sprintf(tempstr, "game%d", i);
        game = filesini->readString("general", tempstr);
        if( game == NULL ) {
            break;
        }
        //filesini->seekSection(game);
        try {
            for( keynum = 1; ;keynum++ ) {
                sprintf(tempstr, "required%d", keynum);
                fname = filesini->readString(game, tempstr);
                if( fname == NULL ) {
                    delete[] fname;
                    break;
                }
                if( !plugins->loadArchive(fname) ) {
                    logger->error("Missing required file \"%s\"\n", fname);
                    delete[] fname;
                    throw 0;
                }
                delete[] fname;
            }
        } catch(int) {
            delete[] game;
            delete plugins;
            continue;
        }
        for( keynum = 1; ;keynum++ ) {
            sprintf(tempstr, "optional%d", keynum);
            fname = filesini->readString(game, tempstr);
            if( fname == NULL ) {
                delete[] fname;
                break;
            }
            plugins->loadArchive(fname);
            delete[] fname;
        }
        delete[] game;
        delete filesini;
        return;
    }

    delete filesini;
    logger->error("Unable to find mixes for any of the specified games!\n");
    exit(1);
}

void VFS_Destroy()
{
    delete plugins;
    delete externals;
}

VFile *VFS_Open(const char *fname)
{
    return VFS_Open(fname,"rb");
}

VFile *VFS_Open(const char *fname, const char* mode)
{
    Uint32 fnum;
    fnum = externals->getFile(fname, mode);
    if( fnum != (Uint32)-1 ) {
        return new VFile(fnum, externals);
    }
    // Won't attempt to write/create files in real archives
    if (mode[0] != 'r') {
        return NULL;
    }
    return plugins->openFile(fname);
}

void VFS_Close(VFile *file)
{
    delete file;
}

const char* VFS_getFirstExisting(const std::vector<const char*>& files)
{
    VFile* tmp;
    for (Uint32 i=0;i<files.size();++i) {
        tmp = VFS_Open(files[i],"r");
        if (tmp != NULL) {
            VFS_Close(tmp);
            return files[i];
        }
    }
    return NULL;
}

const char* VFS_getFirstExisting(Uint32 count, ...)
{
    VFile* tmp;
    va_list ap;
    char* name;
    va_start(ap,count);
    while (count--) {
        name = (char*)va_arg(ap,char*);
        tmp = VFS_Open(name);
        if (tmp != NULL) {
            VFS_Close(tmp);
            va_end(ap);
            return name;
        }
    }
    va_end(ap);
    return NULL;
}
