
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "audengine.h"

const int AudEngine::Steps[89]= {
	7,8,9,10,11,12,13,14,16,17,19,21,23,25,28,31,
	34,37,41,45,50,55,60,66,73,80,88,97,107,118,130,143,
	157,173,190,209,230,253,279,307,337,371,408,449,494,544,598,658,
	724,796,876,963,1060,1166,1282,1411,1552,1707,1878,2066,2272,2499,
	2749,3024,3327,3660,4026,4428,4871,5358,5894,6484,7132,7845,8630,
	9493,10442,11487,12635,13899,15289,16818,18500,20350,22385,24623,
	27086,29794,32767 };
const int AudEngine::Indexes[8] = { -1,-1,-1,-1,2,4,6,8 };


/*  
 * checks fname to see if it is an aud file
 * returns 1 or 0 depending if file is usable
 */
int AudEngine::CheckFile(char *fname)
{
	Uint8 t;
	int ok;
	sndfile = fopen(fname, "rb");
	if( sndfile == NULL ){
		fprintf(stderr, "Unable to load soundfile: %s\n", fname);
		// need to throw a err here
	}
	// skip header
	fseek(sndfile, 11, SEEK_SET);
	
	// set sample and index to 0, these are used in the
	// Decompression alogorithm
	index = 0;
	sample = 0;

	/* check the compression type and throw an error if it's of
		unsuported or unknown type */
	fread(&t, 1, 1, sndfile);
	
	switch (t){
		case 1:
			// TODO see about supporting WW Compressed format
			// (assuming some aud files need it)
			fprintf(stderr, "%s is a WW Compressed format. xmms-aud does not support that format yet.\n", fname);
			ok = 0;
			// need to throw err here
		break;
		case 99:
			// this we can use
			ok = 1;
		break;
		default:
			fprintf(stderr, "%s has a corrupt header.\n", fname);
			ok = 0;
			// throw err here
		break;
	}
	offset = 12;
	
	// set filesize
	fseek(sndfile, 0, SEEK_END);
	filesize = ftell(sndfile);
	//fprintf(stderr, "filesize: %i\n", filesize);
	
	length = (int)((double)filesize / 11212.7);
	//fprintf(stderr, "length: %i\n", length);

	return ok;
}

/*
 * 
 * 
 */
void AudEngine::SetFile(char *sndfile1)
{
	if (!AudEngine::CheckFile(sndfile1)){
		return;
	}

	outBuf = new Uint8[MAX_UNCOMPRESSED_SIZE];
}

/*
 *
 */
void AudEngine::Close(void)
{
	fclose(sndfile);
}

/*
 * 
 * 
 */
Uint16 AudEngine::decodeSample(Uint8 **data)
{
	int i;
	Uint16 size, outSize, len;
	Uint32 ID;
	Uint8 *dataFile;

	*data = outBuf;

	/* allocate some space we can read the compressed data to */
	dataFile = new Uint8[MAX_CHUNK_SIZE];
	len = 0;

	fseek(sndfile, offset, SEEK_SET);

	/* decode 8 samples */
	for( i = 0; i < 8; i++ ) {
		/* abort if we don't have a whole header left */
		if( (offset+8) >= filesize ) {
			delete[] dataFile;
			return len;
		}

		/* read the size, output size and id */
		size = freadword(sndfile);
		outSize = freadword(sndfile);
		ID = freadlong(sndfile);
		/* abort if id was wrong */
		if( ID != 0xDEAF ) {
			delete[] dataFile;
			return len;
		}
		/* read all of the compressed data */
		fread( dataFile, size, 1, sndfile );
		/* decompress it and change to offset and length */
		AudEngine::IMADecode(dataFile, outBuf + len, size, &index, &sample);
		len += outSize;
		offset += size + 8;
	}
	delete[] dataFile;
	return len;
}

/** Decodes IMA-ADPCM Sample
 * @param pointer to compressed sample
 * @param pointer to malloc'd buffer ready to accept decompressed sample
 * @param size in bytes of compressed sample
 * @param current index.
 * @param current samplevalue.
 */
void AudEngine::IMADecode(Uint8 *InBuf, Uint8 *OutBuf, Uint16 Size, Sint32 *index, Sint32 *sample)
{
	int  Samples;
	Uint8 Code;
	int  Sign;
	int  Delta;
	Uint8 *InP;
	Sint16 *OutP;
	Uint16 decSize = Size<<1;

	if (Size==0) {
		return;
	}

	InP=(Uint8 *)InBuf;
	OutP=(Sint16 *)OutBuf;

	for (Samples=0; Samples<decSize; Samples++) {
		if (Samples&1) {		// If Samples is odd
			Code=(*InP++)>>4;	// Extract upper 4 bits
		} else {			// Samples is even
			Code=(*InP) & 0x0F;	// Extract lower 4 bits
		}

		Sign=(Code & 0x08)!=0;		// If topmost bit is set, Sign=true
		Code&=0x07;			// Keep lower 3 bits

		Delta=0;
		if ((Code & 0x04)!=0) {
			Delta+=Steps[*index];
		}
		
		if ((Code & 0x02)!=0) {
			Delta+=Steps[*index]>>1;
		}
		
		if ((Code & 0x01)!=0) {
			Delta+=Steps[*index]>>2;
		}
		
		Delta+=Steps[*index]>>3;

		if (Sign) {
			Delta=-Delta;
		}
		*sample+=Delta;

		if (*sample>32767) {
			*sample=32767;
		} else if (*sample<-32768) {
			*sample=-32768;
		}

		*OutP++=(Sint16)*sample;

		*index+=Indexes[Code];
		if (*index<0) {
			*index=0;
		} else if (*index>88) {
			*index=88;
		}
	}
}
