
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cstring>
#include <pthread.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <xmms/plugin.h>
#include <xmms/configfile.h>
#include <xmms/util.h>
#include "xmms-aud.h"
#include "audengine.h"

AudEngine* music;
static void play_file(char *filename);
static void stop_file(void);
static void get_song_info(char *filename, char **title, int *length);
static int get_time(void);
static void seek (int time);
static void pause_file(short p);
static void plugin_about();
static void about_cb();
static int is_our_file(char *filename);
static GtkWidget *w_about_window;

static pthread_t decode_thread;
static gboolean going = FALSE;
static gboolean seekin = FALSE;
InputPlugin aud_ip = {
	NULL,
	NULL,
	"Aud Player",			/* The description */
	NULL,				/* Called when the plugin is loaded */
	plugin_about,			/* Show the about box */
	NULL,				/* configure */
	is_our_file,			/* Return 1 if the plugin can handle the file*/
	NULL,
	play_file,			/* Guess what... */
	stop_file,			/* Tricky one */
	pause_file,			/* Pause or unpause */
	seek,				/* Seek to the specified time */
	NULL,				/* Set the equalizer */
	get_time,			/* Get the time */
	NULL,				/* Input-plugin specific volume functions */
	NULL,				/* set_volume */
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	get_song_info,			/* Function to grab the title string */
	NULL,				/* Bring up an info window */
	NULL
};


static int get_time(void)
{
	if(going && aud_ip.output->buffer_playing()) {
		return aud_ip.output->output_time();
	} else {
		return -1;
	}
}

static void get_song_info(char *filename, char **title, int *length)
{
	char *name = (char *)malloc(255);
	char *tmpptr = (char *)malloc(255);
	AudEngine *tmp;
	
	tmp = new AudEngine();
	tmp->SetFile(filename);
	(*length) = tmp->length * 1000;
	
	strcpy((char *)name, (char *)filename);
	tmpptr = strrchr((char *)name, '/');
	strcpy((char *)name, (char *)tmpptr);
	memmove(name, name + 1, strlen(name) - 1);
	*strrchr((char *)name, '.') = '\0';
	
	(*title) = (char *)name;
	tmp->Close();
	xmms_usleep(100);
	name = NULL;
	tmpptr = NULL;
	tmp = NULL;
	delete tmp, name, tmpptr;
}

static void seek (int time)
{
	int off2;
	int i;
	
	off2 = music->filesize * time / music->length;
	
	if (off2 <= 12) {
		music->offset = 12;
	} else {
		for (i=12; i < off2; i += 10) {
		}
		music->offset = i - 10;
	}
	seekin = TRUE;
	aud_ip.output->flush((int)time * 1000);
}

static void *play_loop(void *arg)
{
	Uint8 *sndout;
	Uint16 len;
	
	aud_ip.output->open_audio(FMT_S16_LE, 22050, 1);
	
	while (going) {
		len = music->decodeSample(&sndout);
		aud_ip.add_vis_pcm(aud_ip.output->written_time(), FMT_S16_LE, 1, len, sndout);
		
		while (aud_ip.output->buffer_free() < len && going && !seekin) {
			xmms_usleep(4000);
		}
		
		if (seekin) {
			len = music->decodeSample(&sndout);
			aud_ip.output->write_audio(sndout, len);
		}
		
		if (len == 0) {
			break;
		} else {
			if (!seekin) {
				aud_ip.output->write_audio(sndout, len);
			} else {
				seekin = FALSE;
			}
		}
	}
	
	while ((aud_ip.output->output_time() != aud_ip.output->written_time()) && going) {
		xmms_usleep(300);
	}

	aud_ip.output->close_audio();
	music->Close();
	music = NULL;
	going = FALSE;
	xmms_usleep(100);
	pthread_exit(NULL);
}

static void play_file(char *filename)
{
	char *name = (char *)malloc(255);
	char *tmpptr = (char *)malloc(255);	

	music = new AudEngine();
	music->SetFile(filename);
	
	strcpy((char *)name, (char *)filename);
	tmpptr = strrchr((char *)name, '/');
	strcpy((char *)name, (char *)tmpptr);
	memmove(name, name + 1, strlen(name) - 1);
	*strrchr((char *)name, '.') = '\0';

	going = TRUE;
	aud_ip.set_info(name, music->length * 1000, 16384, 22050, 1);
	pthread_create(&decode_thread, NULL, play_loop, NULL);
	
	name = NULL;
	tmpptr = NULL;
	delete name, tmpptr;
}

static void stop_file(void)
{
	if (going) {
		going = FALSE;
		pthread_join(decode_thread, NULL);
	}
}

static void pause_file(short p)
{
	aud_ip.output->pause(p);
}

static void plugin_about(void)
{
	gchar *text =
		"xmms-aud version 0.0.1\n"
		"AUD input plugin for XMMS\n\n"
		"Written By: Bret Towe\n"
		"Email: bret@voxnet.dyndns.org";

	if (!w_about_window) {
		w_about_window = xmms_show_message("About AUD input plugin for XMMS ",
			text, "Close", TRUE,
			about_cb, NULL);
		gtk_widget_show(w_about_window);
	} else {
		gdk_window_raise(w_about_window->window);
	}
}

static void about_cb()
{
	gtk_widget_destroy (w_about_window);
	w_about_window = NULL;
}

static int is_our_file(char *filename)
{
	char *ext;
	// fprintf(stderr, "filename: %s\n",filename);

	ext = strrchr(filename, '.');
	if (ext) {
		if ((!strcasecmp(ext, ".aud")) || (!strcasecmp(ext, ".AUD"))) {
			return 1;
		} else {
			return 0;
		}
	} else {
		return 0;
	}

}

// extern "C" makes it so xmms can see this plugin
// since its written in c++ the functions are seen
// differently in c
// http://gcc.gnu.org/ml/gcc/1998-04/msg00105.html
extern "C" InputPlugin *get_iplugin_info(void)
{
	// aud_ip.description = "AUD Player 0.0.1";
	return &aud_ip;
}
